
/* +---------------------------------------------------------------------+
      M16C-port by Jens Mundhenke, 16.05.05                             
   |   - this module was the former main.c. In this version the uip-     |
   |     stuff acts in background, only a periodic call is needed        |
      - the function uipmain_init has to be called once                 
        to init modules and network interface                           
      - the function uipmain_tick should be called 20 times a second    
        and handles the uip_periodic-calls                              
      - the function uipmain_read is call by the 'packet received'      
        interrupt of the etherdev-device driver                         
                                                                        
      IMHO the software should be unstable, etherdev_send is call from  
      two different interrupts. But it runs and runs very fine.         
      But the interrupt handling is really only a first hack...         
                                                                        
                                                                        
   +---------------------------------------------------------------------+  */

/*  +---------------------------------------------------------------------+
      THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS  
      OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED  
      WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
      ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY    
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
      DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE  
      GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS      
      INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,       
      WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING          
      NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
      SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.       
    +---------------------------------------------------------------------+  */

#include "uip.h"
#include "uip_main.h"
#include "uip_arp.h"
#include "etherdev.h"

unsigned int arptimer = 0;

/*  +---------------------------------------------------------------------+
                                                                         
                                      uipmain_init()                     
                                                                         
     Does everything to make the uip-stack running                       
                                                                         
    | No loop is needed, an incoming packet releases an interrupt,        |
    | the uipmain_tick can be called by an external timer tick            |
    +---------------------------------------------------------------------+  */

int uipmain_init(void)
{
    /* Initialise the uIP TCP/IP stack. */
    uip_init();

    /* Initialise the app. */
    inetd_init();

    /* Initialise the device driver. */
    if ( etherdev_init() )
      return 1;

    /* Initialise the ARP cache. */
    uip_arp_init();
    arptimer = 0;

    return 0;
}


/*
 * uip_tick is called 20 times a second and handles the periodic actions
 */
void uipmain_tick (void)
{
  char i;

  for(i = 0; i < UIP_CONNS; i++) {
    uip_periodic(i);
    if(uip_len > 0) {
      uip_arp_out();
      etherdev_send();
    }
  }

#if UIP_UDP
  for(i = 0; i < UIP_UDP_CONNS; i++) {
    uip_udp_periodic(i);
    if(uip_len > 0) {
      uip_arp_out();
      etherdev_send();
    }
  }
#endif /* UIP_UDP */

  /* Call the ARP timer function every 10 seconds. */
  if(++arptimer >= 20*10) {
    uip_arp_timer();
    arptimer = 0;
  }
}


/*  +---------------------------------------------------------------------+
                                                                         
                               uipmain_read()                            
                                                                         
     Reads a packet and processes it in the uip-way, see manual          
                                                                         
    +---------------------------------------------------------------------+  */
#define BUF ((struct uip_eth_hdr far *)&uip_buf[0])

void uipmain_read(void)
{
  while (uip_len = etherdev_read()) {
    if(BUF->type == htons(UIP_ETHTYPE_IP)) {
      uip_arp_ipin();
      uip_input();
      if(uip_len > 0) {
        uip_arp_out();
        etherdev_send();
      }
    }
    else if(BUF->type == htons(UIP_ETHTYPE_ARP)) {
      uip_arp_arpin();
      if(uip_len > 0)
        etherdev_send();
    }
  }
}

